/*
    DB_DEMO3.PRG

    This demo demonstrates how to add a title bar, which displays the
    database alias and the record number. 

    Note the following:

    1.  cTitleColor is the color of the title line
    2.  aTitleScn is the screen portion overwritten by the title line.
        It is saved so that it can be restored when the db exits.
    3.  bDispBegin is a codeblock EVALuated by ADdbview() just before
        displaying the initial browse screen.
    4.  bDispEnd is a codeblock EVALuated right after displaying the
        initial browse screen.
    5.  bUndisplay is a codeblock EVALuated just before exiting the
        browse.
    6.  These code blocks are made known to ADdbview() via the API,
        ADdb_extra().
    7.  Xdisp_stat() displays the title line.  It is called once by
        the codeblock, bDispEnd, and everytime the browse cursor
        moves vertically.  ADdbview() is advised to these latter
        type of calls via the API ADdb_move().
    8.  The API messages are sent to ADdbview() via the Configuration
        Specifier, [bConfig].

    See also the files:

    SYNTAX.DB - for an explanation of [bConfig].
    CONFIG.DB - for a full description of the APIs.
*/

#include "browse.ch"

//---------------
func main( cDbf )
local cTitleColor, aTitleScn
local bDispBegin := {|e| dispbegin()}

// Displays a title bar at row 24
local bDispEnd := {|e| cTitleColor := if( iscolor(), "GR+/W", "N/W" ),;
                       aTitleScn := ADbox( 24,0,24,79, cTitleColor, "        ", .f., .f. ),;
                       Xdisp_title( cTitleColor ),;  // displays alias and record number
                       dispend();
                  }

// Restores the screen covered by the title screen
local bUndisplay := {|e| ADrestscn( aTitleScn )}
local bConfig := {|e| ADdb_extra( e, bDispBegin, bDispEnd, bUndisplay ),;
                      ADdb_move( e, {|e| Xdisp_title( cTitleColor )} );
                 }


if cDbf = nil .or. !file( cDbf + ".dbf" )
    use demo
else
    use ( cDbf )
endif

ADdbview( 0,0,23,79, bConfig )
return nil


//-----------------------------
func Xdisp_title( cTitleColor )
local bRecno := {|| " " + padr( alltrim( str( recno() ) ) + "/" + alltrim( str( lastrec() ) ), 15 )}
local nRow := row(), nCol := col()

setpos( 24, 1 )
dispout( alias() + eval( bRecno ), cTitleColor )
setpos( nRow, nCol )
return nil
