/*
    DB_DEMO6.PRG

    This demo shows how to implement a horizontal scroll bar.  The
    bar displays the relative position of the browse cursor within the
    series of columns.

    Note the following:

    1.  nSBXEngine is a horizontal scroll bar engine created by
        ADsbx_create().  It is subsequently "operated on"  by the
        SBX APIs, ADsbx_display(), ADsbx_location(), ADsbx_position(),
        and ADsbx_coords().  The SBX is not described here.
    2.  bDispEnd is a codeblock EVALuated by the db engine right after
        the browse is initially displayed.  In this program, it creates
        the SBX engine, displays the scroll bar, and defines a mouse
        hot spot to work in conjucntion with the scroll bar.  This
        codeblock is made known to the db engine via the API,
        ADdb_extra().
    3.  Xlbuts6() is a function that processes the mouse clicks.  Its
        existence is made known to the db engine via the API,
        ADdb_lbuttons().
    4.  Xxmove6() s a function that is called whenever the browse
        cursor changes column.  In this demo, it updates the location
        of the scroll character.  Its existence is made known to the
        DB engine via the API, ADdb_move().
    5.  The API messages are sent to ADdbview() via the Configuration
        Specifier, [bConfig].

    See also the files:

    SYNTAX.DB - for an explanation of [bConfig].
    CONFIG.DB - for a full description of the APIs.
*/


//---------------
func main( cDbf )
local nSBXEngine    // horizontal scroll bar engine
local bDispBegin, bUndisplay
local bDispEnd := { |e| nSBXEngine := ADsbx_create(),;
                        ADsbx_location( e, ADdb_bottom(e), ADdb_left(e) + 1, ADdb_object(e):colcount ),;
                        ADsbx_display( nSBXEngine, ( ADdb_object(e) ):colpos, .f. ),;
                        ADdb_lbuttons( e, { ADsbx_coords( nSBXEngine ) },;
                                          {|e,nth,mRow,mCol| Xlbuts6(e,nth,mRow,mCol, nSBXEngine)};
                                     );
                  }
local bConfig := {|e| ADdb_extra(e, bDispBegin, bDispEnd ),;
                      ADdb_move( e,, {|e| Xxmove6(e, nSBXEngine)} );
                 }

if cDbf = nil .or. !file( cDbf + ".dbf" )
    use demo
else
    use ( cDbf )
endif

ADdbview( 4,4,20,76, bConfig )
return nil


//-----------------------------------------
func Xlbuts6(e,nth,mRow,mCol,nSBXEngine)
local nDistance, xx

if nth = 1
    nDistance = ADsbx_position( nSBXEngine, mCol ) - ADdb_object(e):colpos

    if nDistance != 0
        if nDistance < 0
            for xx = nDistance to -1
                ADdb_object(e):left()
            next
        elseif nDistance > 0
            for xx = 1 to nDistance
                ADdb_object(e):right()
            next
        endif

        ADdb_object(e):refreshall()
    endif
endif

return nil


//----------------------------
func Xxmove6(e, nSBXEngine)
ADsbx_display( nSBXEngine, ( ADdb_object(e) ):colpos )
return nil

