/*
    DB_DEMO7.PRG

    This demo shows how to implement a vertical scroll bar.  The
    bar displays the relative record position.

    Note the following:

    1.  nSBYEngine is a vertical scroll bar engine created by
        ADsby_create().  It is subsequently "operated on"  by the
        SBY APIs, ADsby_display(), ADsby_location(), ADsby_position(),
        and ADsby_coords().  The SBY is not described here.
    2.  bDispEnd is a codeblock EVALuated by the db engine right after
        the browse is initially displayed.  In this program, it creates
        the SBX engine, displays the scroll bar, and defines a mouse
        hot spot to work in conjucntion with the scroll bar.  This
        codeblock is made known to the db engine via the API,
        ADdb_extra().
    3.  Xlbuts7() is a function that processes the mouse clicks.  Its
        existence is made known to the db engine via the API,
        ADdb_lbuttons().
    4.  Xymove7() s a function that is called whenever the browse
        cursor moves vertically.  In this demo, it updates the location
        of the scroll character.  Its existence is made known to the
        DB engine via the API, ADdb_move().
    5.  The API messages are sent to ADdbview() via the Configuration
        Specifier, [bConfig].

    See also the files:

    SYNTAX.DB - for an explanation of [bConfig].
    CONFIG.DB - for a full description of the APIs.
*/

#include "browse.ch"

#define SBYENGINE       ADdb_cargo(e)[1]
#define SBYPOSITIONS    ADdb_cargo(e)[2]
#define SBYUNITPOSITION ADdb_cargo(e)[3]
#define LOGNO           ADdb_cargo(e)[4]
#define CARGO_SIZE      4

//---------------
func main( cDbf )
local nLogno, nLength
local bDispBegin
local bDispEnd := { |e| ADdb_cargo(e, array( CARGO_SIZE ) ),;
                        LOGNO := nLogno := ADlogno(),;
                        nLength := min( ADdb_object(e):rowcount, ADround( lastrec(), ADdb_object(e):rowcount ) ),;
                        SBYENGINE := ADsby_create(),;
                        ADsby_location( SBYENGINE, ADdb_top(e) + ADdb_headlines(e) + 2, ADdb_right(e), nLength ),;
                        SBYPOSITIONS := array( nLength, 2 ),;
                        SBYUNITPOSITION := ADround( lastrec(), nLength ),;
                        Xcalc_sbyunitpositions( e ),;
                        ADsby_display( SBYENGINE, ascan( SBYPOSITIONS, {|x| nLogno >= x[1] .and. nLogno <= x[2]} ), SB_INITIALIZE ),;
                        ADdb_lbuttons( e, {;
                                            ADsby_coords( SBYENGINE );
                                          };
                                     );
                  }
local bUndisplay
local bConfig := {|e| ADdb_lbuttons( e,, {|e,nth,mRow,mCol| Xlbuts7(e,nth,mRow,mCol)} ),;
                      ADdb_move( e, {|e| Xymove7(e)} ),;
                      ADdb_extra( e, bDispBegin, bDispEnd, bUndisplay );
                 }

if cDbf = nil .or. !file( cDbf + ".dbf" )
    use demo
else
    use ( cDbf )
endif

if lastrec() < 50
    ADmessage( { "Use a database with at least 50 records so the scroll",;
                 "bar may be thoroughly demonstrated." } )
else
    ADdbview( 10,10,20,69, bConfig )
endif
return nil


//---------------------------
func Xlbuts7(e,nth,mRow,mCol)
local nDistance, xx

    LOGNO = SBYPOSITIONS[ADsby_position( SBYENGINE, mRow )][1]
    go top
    skip LOGNO - 1
    ADdb_object(e):refreshall()
    ADdb_vertmove(e, DB_VERTMOVE_UNKNOWN )

return nil

//-------------
func Xymove7(e)
local nVertMove := ADdb_vertmove(e)
local nLogno := LOGNO

if nVertmove = DB_VERTMOVE_UNKNOWN
    ADsby_display( SBYENGINE, ascan( SBYPOSITIONS, {|x| LOGNO >= x[1] .and. LOGNO <= x[2]} ) )
elseif nVertmove = DB_VERTMOVE_BOTTOM
    LOGNO = lastrec()
elseif nVertmove = DB_VERTMOVE_TOP
    LOGNO = 1
else
    LOGNO += nVertmove

    if LOGNO < 1
        LOGNO = 1
    elseif LOGNO > lastrec()
        LOGNO = lastrec()
    endif
endif

if nLogno != LOGNO
    ADsby_display( SBYENGINE, ascan( SBYPOSITIONS, {|x| LOGNO >= x[1] .and. LOGNO <= x[2]} ) )
endif
return nil


//----------------------------------
func Xcalc_sbyunitpositions( e )
local xx

SBYPOSITIONS[1][1] = 1

for xx = 1 to len( SBYPOSITIONS ) - 1
    SBYPOSITIONS[xx][2] = SBYPOSITIONS[xx][1] + SBYUNITPOSITION - 1
    SBYPOSITIONS[xx+1][1] = SBYPOSITIONS[xx][2] + 1
next

SBYPOSITIONS[len (SBYPOSITIONS) ][2] = lastrec()
return nil


